searchData={"items":[{"type":"module","title":"JSONAPIPlug","doc":"JSONAPIPlug context\n\nThis defines a struct for storing configuration and request data. `JSONAPIPlug.Plug` populates\nits attributes by means of a number of other plug modules used to parse and validate requests\nand stores it in the `Plug.Conn` private assings under the `jsonapi_plug` key.","ref":"JSONAPIPlug.html"},{"type":"function","title":"JSONAPIPlug.mime_type/0","doc":"JSON:API MIME type\n\nReturns the JSON:API MIME type.","ref":"JSONAPIPlug.html#mime_type/0"},{"type":"function","title":"JSONAPIPlug.recase/2","doc":"Recase resource fields\n\nChanges the case of resource field names to the specified case, ignoring underscores\nor dashes that are not between letters/numbers.","ref":"JSONAPIPlug.html#recase/2"},{"type":"function","title":"Examples - JSONAPIPlug.recase/2","doc":"iex> recase(\"top_posts\", :camelize)\n    \"topPosts\"\n\n    iex> recase(:top_posts, :camelize)\n    \"topPosts\"\n\n    iex> recase(\"_top_posts\", :camelize)\n    \"_topPosts\"\n\n    iex> recase(\"_top__posts_\", :camelize)\n    \"_top__posts_\"\n\n    iex> recase(\"\", :camelize)\n    \"\"\n\n    iex> recase(\"top_posts\", :dasherize)\n    \"top-posts\"\n\n    iex> recase(\"_top_posts\", :dasherize)\n    \"_top-posts\"\n\n    iex> recase(\"_top__posts_\", :dasherize)\n    \"_top__posts_\"\n\n    iex> recase(\"top-posts\", :underscore)\n    \"top_posts\"\n\n    iex> recase(:top_posts, :underscore)\n    \"top_posts\"\n\n    iex> recase(\"-top-posts\", :underscore)\n    \"-top_posts\"\n\n    iex> recase(\"-top--posts-\", :underscore)\n    \"-top--posts-\"\n\n    iex> recase(\"corgiAge\", :underscore)\n    \"corgi_age\"","ref":"JSONAPIPlug.html#recase/2-examples"},{"type":"function","title":"JSONAPIPlug.render/5","doc":"Render JSON:API response\n\nRenders the JSON:API response for the specified Resource.","ref":"JSONAPIPlug.html#render/5"},{"type":"function","title":"JSONAPIPlug.url_for/2","doc":"Generates the resource link\n\nGenerates the resource link for a resource.","ref":"JSONAPIPlug.html#url_for/2"},{"type":"function","title":"JSONAPIPlug.url_for_relationship/3","doc":"Generate relationships link\n\nGenerates the relationships link for a resource.","ref":"JSONAPIPlug.html#url_for_relationship/3"},{"type":"type","title":"JSONAPIPlug.case/0","doc":"String case","ref":"JSONAPIPlug.html#t:case/0"},{"type":"type","title":"JSONAPIPlug.t/0","doc":"JSONAPIPlug context","ref":"JSONAPIPlug.html#t:t/0"},{"type":"module","title":"JSONAPIPlug.API","doc":"JSON:API API Configuration\n\n  You can define an API by \"use-ing\" `JSONAPIPlug.API` in your API module:\n\n  ```elixir\n  defmodule MyApp.API do\n    use JSONAPIPlug.API, otp_app: :my_app\n  end\n  ```\n\n  API module configuration can be customized via your application configuration:\n\n  ```elixir\n  config :my_app, MyApp.API,\n    namespace: \"api\",\n    case: :dasherize\n  ```\n\n  See `t:options/0` for all available configuration options.","ref":"JSONAPIPlug.API.html"},{"type":"function","title":"JSONAPIPlug.API.get_config/1","doc":"Retrieve API configuration\n\nPlease note that API configuration is also cached on first request and read back from it afterwards.","ref":"JSONAPIPlug.API.html#get_config/1"},{"type":"type","title":"JSONAPIPlug.API.options/0","doc":"API configuration options:\n\n* `:case` - This option controls how your API's field names will be cased. The current `JSON:API Specification v1.0` recommends dasherizing (e.g. `\"favorite-color\": \"blue\"`), while the upcoming `JSON:API Specification v1.1` recommends camelCase (e.g. `\"favoriteColor\": \"blue\"`). The default value is `:camelize`.\n\n* `:client_generated_ids` (`t:boolean/0`) - Enable support for Client-Generated IDs. When enabled, the resources received in requests are supposed to contain a valid 'id'. The default value is `false`.\n\n* `:host` (`t:String.t/0`) - Hostname used for link generation instead of deriving it from the connection.\n\n* `:namespace` (`t:String.t/0`) - Namespace for all resources in your API. if you want your resources to live under \".../api/v1\", pass `namespace: \"api/v1\"`.\n\n* `:normalizer` (`t:atom/0`) - Normalizer for transformation of `JSON:API` document to and from user data. The default value is `JSONAPIPlug.Normalizer.Ecto`.\n\n* `:query_parsers` (`t:keyword/0`) - Parsers for transformation of `JSON:API` request query parameters to user data. The default value is `[fields: JSONAPIPlug.QueryParser.Ecto.Fields, filter: JSONAPIPlug.QueryParser.Filter, include: JSONAPIPlug.QueryParser.Ecto.Include, page: JSONAPIPlug.QueryParser.Page, sort: JSONAPIPlug.QueryParser.Ecto.Sort]`.\n\n  * `:fields` (`t:atom/0`) - Fields query parameter parser. The default value is `JSONAPIPlug.QueryParser.Ecto.Fields`.\n\n  * `:filter` (`t:atom/0`) - Filter query parameter parser. The default value is `JSONAPIPlug.QueryParser.Filter`.\n\n  * `:include` (`t:atom/0`) - Include query parameter parser. The default value is `JSONAPIPlug.QueryParser.Ecto.Include`.\n\n  * `:page` (`t:atom/0`) - Page query parameter parser. The default value is `JSONAPIPlug.QueryParser.Page`.\n\n  * `:sort` (`t:atom/0`) - Sort query parameter parser. The default value is `JSONAPIPlug.QueryParser.Ecto.Sort`.\n\n* `:pagination` (`t:atom/0`) - A module adopting the `JSONAPIPlug.Pagination` behaviour for pagination. The default value is `nil`.\n\n* `:port` (`t:pos_integer/0`) - Port used for link generation instead of deriving it from the connection.\n\n* `:scheme` - Scheme used for link generation instead of deriving it from the connection.\n\n* `:version` - `JSON:API` version advertised in the document The default value is `:\"1.0\"`.","ref":"JSONAPIPlug.API.html#t:options/0"},{"type":"type","title":"JSONAPIPlug.API.t/0","doc":"","ref":"JSONAPIPlug.API.html#t:t/0"},{"type":"module","title":"JSONAPIPlug.Phoenix.Component","doc":"JSONAPIPlug Phoenix Component helper","ref":"JSONAPIPlug.Phoenix.Component.html"},{"type":"module","title":"JSONAPIPlug.Plug","doc":"Implements validation and parsing of `JSON:API` requests\n\nThis plug handles the specification defined `JSON:API` request body and query parameters\n(`fields`, `filter`, `include`, `page` and `sort`).","ref":"JSONAPIPlug.Plug.html"},{"type":"module","title":"Usage - JSONAPIPlug.Plug","doc":"Add this plug to your plug pipeline/controller like this:\n\n```\nplug JSONAPIPlug.Plug, api: MyApp.API, resource: MyApp.MyResource\n```\n\nIf your connection receives a valid `JSON:API` request this plug will parse it into a\n`JSONAPIPlug` struct that will be stored in the `Plug.Conn` private assign `:jsonapi_plug`.\nThe final `Plug.Conn` struct will look similar to the following:\n\n```\n%Plug.Conn{\n  ...\n  body_params: %{...},\n  params: %{\"data\" => ...},\n  private: %{\n    ...\n    jsonapi_plug: %JSONAPIPlug{\n      api: MyApp.API,\n      fields: ..., # Defaults to a map of field names by type.\n      filter: ..., # Defaults to the query parameter value.\n      include: ..., # Defaults to Ecto preload format.\n      page: ..., # Defaults to the query parameter value.\n      params: ..., # Defaults to Ecto normalized data.\n      sort: ..., # Defaults to Ecto order_by format.\n      resource: MyApp.MyResource\n    }\n    ...\n  }\n  ...\n}\n```\n\nYou can then use the contents of the struct to load data and call `JSONAPIPlug.render/4`\nor the render function generated by `use JSONAPIPlug.Resource` to generate responses and render\nthem.","ref":"JSONAPIPlug.Plug.html#module-usage"},{"type":"module","title":"Customizing default behaviour - JSONAPIPlug.Plug","doc":"","ref":"JSONAPIPlug.Plug.html#module-customizing-default-behaviour"},{"type":"module","title":"Body parameters - JSONAPIPlug.Plug","doc":"By default, body parameters are transformed into a format that is compatible with attrs for\n`Ecto.Changeset` to perform inserts/updates of `Ecto.Schema` modules. However, you can transform\nthe `JSON:API` document in any format you want by writing your own module adopting the\n`JSONAPIPlug.Normalizer` behaviour and configuring it through `JSONAPIPlug.API` configuration.","ref":"JSONAPIPlug.Plug.html#module-body-parameters"},{"type":"module","title":"Query parameters - JSONAPIPlug.Plug","doc":"The `JSON:API` `fields` and `include` query parameters format is defined by the specification.\nThe default implementation accepts the specification format and converts it to data usable as\n`select` and `preload` options to `Ecto.Repo` functions.\n\nThe `JSON:API` `sort` query parameter format is not defined, however the specification suggests\nto use a format for encoding sorting by attribute names with an optional `-` prefix to invert\nordering direction. The default implementation accepts the suggested format and converts it to\nusable as `order_by` option to `Ecto.Repo` functions.\n\nThe `JSON:API` `filter` and `page` query parameters format is not defined by the JSON:API specification,\ntherefore the default implementation just copies the value of the query parameters in `JSONAPIPlug`.\n\nYou can transform data in any format you want for any of these parameters by implementing a module\nadopting the `JSONAPIPlug.QueryParser` behaviour and configuring it through `JSONAPIPlug.API` configuration.","ref":"JSONAPIPlug.Plug.html#module-query-parameters"},{"type":"function","title":"JSONAPIPlug.Plug.call/2","doc":"","ref":"JSONAPIPlug.Plug.html#call/2"},{"type":"type","title":"JSONAPIPlug.Plug.options/0","doc":"Options:\n* `:api` (`t:atom/0`) - Required. A module use-ing `JSONAPIPlug.API` to provide configuration\n\n* `:includes` (`t:keyword/0`) - A nested keyword list of allowed includes for this endpoint. By default all includes are allowed. To disable all include, pass `[]`.\n\n* `:normalizer` (`t:atom/0`) - Normalizer module. If unspecified, defaults to the API configured one.\n\n* `:path` (`t:String.t/0`) - Resource path. The resource type is used unless speficied.\n\n* `:resource` (`t:atom/0`) - Required. The `JSONAPIPlug.Resource` used to parse the request.","ref":"JSONAPIPlug.Plug.html#t:options/0"},{"type":"module","title":"JSONAPIPlug.Document","doc":"JSON:API Document\n\nThis module defines the structure of a `JSON:API` document and functions that handle\nserialization and deserialization. This also handles validation of `JSON:API` documents.\n\nhttps://jsonapi.org/format/#document-structure","ref":"JSONAPIPlug.Document.html"},{"type":"function","title":"JSONAPIPlug.Document.deserialize/1","doc":"Deserialize JSON:API Document\n\nTakes a map representing a JSON:API Document as input, validates it\nand parses it into a `t:t/0` struct.","ref":"JSONAPIPlug.Document.html#deserialize/1"},{"type":"function","title":"JSONAPIPlug.Document.serialize/1","doc":"Serialize a Document struct representing a JSON:API Document\n\nTakes a `t:t/0` struct representing a JSON:API Document as input, validates\nit and returns the struct if valid.","ref":"JSONAPIPlug.Document.html#serialize/1"},{"type":"type","title":"JSONAPIPlug.Document.data/0","doc":"JSON:API Primary Data\n\nhttps://jsonapi.org/format/#document-top-level","ref":"JSONAPIPlug.Document.html#t:data/0"},{"type":"type","title":"JSONAPIPlug.Document.errors/0","doc":"JSON:API Errors\n\nhttps://jsonapi.org/format/#errors","ref":"JSONAPIPlug.Document.html#t:errors/0"},{"type":"type","title":"JSONAPIPlug.Document.included/0","doc":"JSON:API Included Resources\n\nhttps://jsonapi.org/format/#document-compound-documents","ref":"JSONAPIPlug.Document.html#t:included/0"},{"type":"type","title":"JSONAPIPlug.Document.jsonapi/0","doc":"JSON:API Object\n\nhttps://jsonapi.org/format/#document-jsonapi-object","ref":"JSONAPIPlug.Document.html#t:jsonapi/0"},{"type":"type","title":"JSONAPIPlug.Document.links/0","doc":"JSON:API Links\n\nhttps://jsonapi.org/format/#document-links","ref":"JSONAPIPlug.Document.html#t:links/0"},{"type":"type","title":"JSONAPIPlug.Document.meta/0","doc":"JSON:API Meta Information\n\nhttps://jsonapi.org/format/#document-meta","ref":"JSONAPIPlug.Document.html#t:meta/0"},{"type":"type","title":"JSONAPIPlug.Document.payload/0","doc":"","ref":"JSONAPIPlug.Document.html#t:payload/0"},{"type":"type","title":"JSONAPIPlug.Document.t/0","doc":"JSON:API Document\n\nhttps://jsonapi.org/format/#document-structure","ref":"JSONAPIPlug.Document.html#t:t/0"},{"type":"type","title":"JSONAPIPlug.Document.value/0","doc":"","ref":"JSONAPIPlug.Document.html#t:value/0"},{"type":"module","title":"JSONAPIPlug.Document.ErrorObject","doc":"JSON:API Error Object\n\nhttps://jsonapi.org/format/#error-objects","ref":"JSONAPIPlug.Document.ErrorObject.html"},{"type":"function","title":"JSONAPIPlug.Document.ErrorObject.deserialize/1","doc":"","ref":"JSONAPIPlug.Document.ErrorObject.html#deserialize/1"},{"type":"function","title":"JSONAPIPlug.Document.ErrorObject.serialize/1","doc":"","ref":"JSONAPIPlug.Document.ErrorObject.html#serialize/1"},{"type":"type","title":"JSONAPIPlug.Document.ErrorObject.t/0","doc":"","ref":"JSONAPIPlug.Document.ErrorObject.html#t:t/0"},{"type":"module","title":"JSONAPIPlug.Document.JSONAPIObject","doc":"JSON:API Document JSON:API Object","ref":"JSONAPIPlug.Document.JSONAPIObject.html"},{"type":"function","title":"JSONAPIPlug.Document.JSONAPIObject.deserialize/1","doc":"","ref":"JSONAPIPlug.Document.JSONAPIObject.html#deserialize/1"},{"type":"type","title":"JSONAPIPlug.Document.JSONAPIObject.t/0","doc":"","ref":"JSONAPIPlug.Document.JSONAPIObject.html#t:t/0"},{"type":"type","title":"JSONAPIPlug.Document.JSONAPIObject.version/0","doc":"","ref":"JSONAPIPlug.Document.JSONAPIObject.html#t:version/0"},{"type":"module","title":"JSONAPIPlug.Document.LinkObject","doc":"JSON:API Link Object\n\nhttps://jsonapi.org/format/#document-links","ref":"JSONAPIPlug.Document.LinkObject.html"},{"type":"function","title":"JSONAPIPlug.Document.LinkObject.deserialize/1","doc":"","ref":"JSONAPIPlug.Document.LinkObject.html#deserialize/1"},{"type":"function","title":"JSONAPIPlug.Document.LinkObject.serialize/1","doc":"","ref":"JSONAPIPlug.Document.LinkObject.html#serialize/1"},{"type":"type","title":"JSONAPIPlug.Document.LinkObject.t/0","doc":"","ref":"JSONAPIPlug.Document.LinkObject.html#t:t/0"},{"type":"module","title":"JSONAPIPlug.Document.RelationshipObject","doc":"JSON:API Relationship Object\n\nhttps://jsonapi.org/format/#document-resource-object-relationships","ref":"JSONAPIPlug.Document.RelationshipObject.html"},{"type":"function","title":"JSONAPIPlug.Document.RelationshipObject.deserialize/1","doc":"","ref":"JSONAPIPlug.Document.RelationshipObject.html#deserialize/1"},{"type":"type","title":"JSONAPIPlug.Document.RelationshipObject.links/0","doc":"","ref":"JSONAPIPlug.Document.RelationshipObject.html#t:links/0"},{"type":"type","title":"JSONAPIPlug.Document.RelationshipObject.t/0","doc":"","ref":"JSONAPIPlug.Document.RelationshipObject.html#t:t/0"},{"type":"module","title":"JSONAPIPlug.Document.ResourceIdentifierObject","doc":"JSON:API Resource Identifier object\n\nhttps://jsonapi.org/format/#document-resource-object-linkage","ref":"JSONAPIPlug.Document.ResourceIdentifierObject.html"},{"type":"function","title":"JSONAPIPlug.Document.ResourceIdentifierObject.deserialize/1","doc":"","ref":"JSONAPIPlug.Document.ResourceIdentifierObject.html#deserialize/1"},{"type":"type","title":"JSONAPIPlug.Document.ResourceIdentifierObject.t/0","doc":"","ref":"JSONAPIPlug.Document.ResourceIdentifierObject.html#t:t/0"},{"type":"module","title":"JSONAPIPlug.Document.ResourceObject","doc":"JSON:API Resource Object\n\nhttps://jsonapi.org/format/#resource_object-resource-objects","ref":"JSONAPIPlug.Document.ResourceObject.html"},{"type":"function","title":"JSONAPIPlug.Document.ResourceObject.deserialize/1","doc":"","ref":"JSONAPIPlug.Document.ResourceObject.html#deserialize/1"},{"type":"function","title":"JSONAPIPlug.Document.ResourceObject.serialize/1","doc":"","ref":"JSONAPIPlug.Document.ResourceObject.html#serialize/1"},{"type":"type","title":"JSONAPIPlug.Document.ResourceObject.id/0","doc":"","ref":"JSONAPIPlug.Document.ResourceObject.html#t:id/0"},{"type":"type","title":"JSONAPIPlug.Document.ResourceObject.t/0","doc":"","ref":"JSONAPIPlug.Document.ResourceObject.html#t:t/0"},{"type":"type","title":"JSONAPIPlug.Document.ResourceObject.type/0","doc":"","ref":"JSONAPIPlug.Document.ResourceObject.html#t:type/0"},{"type":"protocol","title":"JSONAPIPlug.Resource","doc":"You can use any struct as a resource by deriving or directly implementing the `JSONAPIPlug.Resource` protocol:\n\n```elixir\ndefmodule MyApp.User do\n  @derive {\n    JSONAPIPlug.Resource,\n    type: \"user\",\n    attributes: [:name, :surname, :username]\n  }\n  defstruct id: nil, name: nil, surname: nil, username: nil\nend\n```\nSee `t:options/0` for all available options you can pass to `@derive JSONAPIPlug.Resource`.","ref":"JSONAPIPlug.Resource.html"},{"type":"protocol","title":"Attributes - JSONAPIPlug.Resource","doc":"By default, the resulting JSON document consists of resources taken from your data.\nOnly resource attributes defined on the resource will be (de)serialized. You can customize\nhow attributes are handled by passing a keyword list of options:\n\n```elixir\ndefmodule MyApp.User do\n  @derive {\n    JSONAPIPlug.Resource,\n    type: \"user\",\n    attributes: [\n      username: nil,\n      fullname: [deserialize: false]\n    ]\n  }\nend\n```\n\nIn this example we are defining a computed attribute by passing the `serialize` option a function reference.\nSerialization functions take `resource` and `conn` as arguments and return the attribute value to be serialized.\nThe `deserialize` option set to `false` makes sure the attribute is not deserialized when receiving a request.","ref":"JSONAPIPlug.Resource.html#module-attributes"},{"type":"protocol","title":"Relationships - JSONAPIPlug.Resource","doc":"Relationships are defined by passing the `relationships` option to `use JSONAPIPlug.Resource`.\n\n```elixir\ndefmodule MyApp.Post do\n  @derive {\n    JSONAPIPlug.Resource,\n    type: \"post\",\n    attributes: [:text, :body]\n    relationships: [\n      author: [resource: MyApp.UsersResource],\n      comments: [many: true, resource: MyApp.CommentsResource]\n    ]\n  }\nend\n\ndefmodule MyApp.CommentsResource do\n  alias MyApp.UsersResource\n\n  @derive {\n    JSONAPIPlug.Resource,\n    type: \"comment\",\n    attributes: [:text]\n    relationships: [post: [resource: MyApp.Post]]\n  }\n  defstruct text: nil, post: nil\nend\n```\n\nWhen requesting `GET /posts?include=author`, if the author key is present on the data you pass from the\ncontroller it will appear in the `included` section of the JSON:API response.","ref":"JSONAPIPlug.Resource.html#module-relationships"},{"type":"protocol","title":"Links - JSONAPIPlug.Resource","doc":"When rendering resource links, the default behaviour is to is to derive values for `host`, `port`\nand `scheme` from the connection. If you need to use different values for some reason, you can override them\nusing `JSONAPIPlug.API` configuration options in your api configuration:\n\n```elixir\nconfig :my_app, MyApp.API, host: \"adifferenthost.com\"\n```","ref":"JSONAPIPlug.Resource.html#module-links"},{"type":"function","title":"JSONAPIPlug.Resource.attributes/1","doc":"Returns the resource attributes","ref":"JSONAPIPlug.Resource.html#attributes/1"},{"type":"function","title":"JSONAPIPlug.Resource.field_option/3","doc":"Resource field option\n\nReturns the value of the requested field option","ref":"JSONAPIPlug.Resource.html#field_option/3"},{"type":"function","title":"JSONAPIPlug.Resource.id_attribute/1","doc":"Resource Id Attribute\n\nReturns the attribute used to fetch resource ids for resources by the","ref":"JSONAPIPlug.Resource.html#id_attribute/1"},{"type":"function","title":"JSONAPIPlug.Resource.recase_field/3","doc":"Resource function to recase fields\n\nReturns the field is the required case","ref":"JSONAPIPlug.Resource.html#recase_field/3"},{"type":"function","title":"JSONAPIPlug.Resource.relationships/1","doc":"Resource relationships\n\nReturns the keyword list of resource relationships for the","ref":"JSONAPIPlug.Resource.html#relationships/1"},{"type":"function","title":"JSONAPIPlug.Resource.type/1","doc":"Resource Type\n\nReturns the Resource Type of resources for the","ref":"JSONAPIPlug.Resource.html#type/1"},{"type":"type","title":"JSONAPIPlug.Resource.field_name/0","doc":"Resource field name\n\nThe name of a Resource field (attribute or relationship)","ref":"JSONAPIPlug.Resource.html#t:field_name/0"},{"type":"type","title":"JSONAPIPlug.Resource.options/0","doc":"Resource options\n\nAvailable options:\n* `:attributes` - Resource attributes. This will be used to (de)serialize requests/responses:\n\n    * `:name` (`t:atom/0`) - Maps the resource attribute name to the given key.\n\n    * `:serialize` (`t:boolean/0`) - Controls wether the attribute is serialized in responses. The default value is `true`.\n\n    * `:deserialize` (`t:boolean/0`) - Controls wether the attribute is deserialized in requests. The default value is `true`.\n\n  The default value is `[]`.\n\n* `:id_attribute` (`t:atom/0`) - Attribute on your data to be used as the JSON:API resource id. The default value is `:id`.\n\n* `:relationships` (`t:keyword/0`) - Resource relationships. This will be used to (de)serialize requests/responses\n\n    * `:name` (`t:atom/0`) - Maps the resource relationship name to the given key.\n\n    * `:many` (`t:boolean/0`) - Specifies a to many relationship. The default value is `false`.\n\n    * `:resource` (`t:atom/0`) - Required. Specifies the resource to be used to serialize the relationship\n\n  The default value is `[]`.\n\n  * `:name` (`t:atom/0`) - Maps the resource relationship name to the given key.\n\n  * `:many` (`t:boolean/0`) - Specifies a to many relationship. The default value is `false`.\n\n  * `:resource` (`t:atom/0`) - Required. Specifies the resource to be used to serialize the relationship\n\n* `:type` (`t:String.t/0`) - Required. Resource type. To be used as the JSON:API resource type value","ref":"JSONAPIPlug.Resource.html#t:options/0"},{"type":"type","title":"JSONAPIPlug.Resource.t/0","doc":"Resource module\n\nA struct implementing the `JSONAPIPlug.Resource` protocol","ref":"JSONAPIPlug.Resource.html#t:t/0"},{"type":"protocol","title":"JSONAPIPlug.Resource.Attribute","doc":"Custom Resource attributes serialization and deserialization\n\nThis protocol allows to customize how individual resource attributes\nare serialized to in responses and deserialized from requests.\n\nThe default implementation serializes and deserializes  attributes as\nthey appear in the resource.\n\nThe implementation for `MyApp.Post` below, serializes the excerpt\nattribute by taking the first 10 characters of the post body value,\nand preserves all other attributes values in other cases.\n\n```elixir\ndefimpl JSONAPIPlug.Resource.Attribute, for: MyApp.Post do\n  def serialize(%@for{}, :excerpt, _value, _conn),\n    do: String.slice(post.body, 0..9)\n\n  def serialize(%@for{}, _attribute, _value, _conn), do: value\n\n  def deserialize(%@for{}, _attribute, _value, _conn), do: value\nend\n```","ref":"JSONAPIPlug.Resource.Attribute.html"},{"type":"function","title":"JSONAPIPlug.Resource.Attribute.deserialize/4","doc":"Customize deserialization of resource attribute value from the request","ref":"JSONAPIPlug.Resource.Attribute.html#deserialize/4"},{"type":"function","title":"JSONAPIPlug.Resource.Attribute.serialize/4","doc":"Customize serialization of resource attribute value in the response","ref":"JSONAPIPlug.Resource.Attribute.html#serialize/4"},{"type":"type","title":"JSONAPIPlug.Resource.Attribute.t/0","doc":"All the types that implement this protocol.","ref":"JSONAPIPlug.Resource.Attribute.html#t:t/0"},{"type":"protocol","title":"JSONAPIPlug.Resource.Links","doc":"Resource Links\n\nImplement this protocol to generate JSON:API links for individual resources.\nLinks returned will be included for each resource `links` in JSON:API responses.\n\nThis example implementation generates a link for each `MyApp.Post` resource.\n\n```elixir\ndefimpl JSONAPIPlug.Resource.Links, for: MyApp.Post do\n  def links(%@for{} = post, _conn), do: %{some-link: \"http://myapp.com/post/#{post.id}/other-stuff\"}\nend\n```","ref":"JSONAPIPlug.Resource.Links.html"},{"type":"function","title":"JSONAPIPlug.Resource.Links.links/2","doc":"Resource links\n\nReturns the resource links to be returned for resources by the resource.","ref":"JSONAPIPlug.Resource.Links.html#links/2"},{"type":"type","title":"JSONAPIPlug.Resource.Links.t/0","doc":"All the types that implement this protocol.","ref":"JSONAPIPlug.Resource.Links.html#t:t/0"},{"type":"protocol","title":"JSONAPIPlug.Resource.Meta","doc":"Resource Links\n\nImplement this protocol to generate JSON:API metadata for indivudual resources.\nMetadata returned will be included for each resource `meta` in JSON:API responses.\n\nThis example implementation generates metadata for each `MyApp.Post` resource.\n\n```elixir\n  defimpl JSONAPIPlug.Resource.Meta, for: MyApp.Post do\n    def meta(%@for{} = post, _conn), do: %{slug: to_slug(post.title)}\n\n    defp to_slug(string), do: string  |> String.downcase |> String.replace(\" \", \"-\")\n  end\n```","ref":"JSONAPIPlug.Resource.Meta.html"},{"type":"function","title":"JSONAPIPlug.Resource.Meta.meta/2","doc":"Resource meta\n\nReturns the resource meta to be returned for resources by the resource.","ref":"JSONAPIPlug.Resource.Meta.html#meta/2"},{"type":"type","title":"JSONAPIPlug.Resource.Meta.t/0","doc":"All the types that implement this protocol.","ref":"JSONAPIPlug.Resource.Meta.html#t:t/0"},{"type":"module","title":"JSONAPIPlug.Plug.ContentTypeNegotiation","doc":"Provides content type negotiation by validating the `content-type` and `accept` headers.\n\nThe proper jsonapi.org content type is `application/vnd.api+json` as per\n[the specification](http://jsonapi.org/format/#content-negotiation-servers).","ref":"JSONAPIPlug.Plug.ContentTypeNegotiation.html"},{"type":"module","title":"JSONAPIPlug.Plug.Params","doc":"Plug for parsing the JSON:API Document in requests\n\nIt parses the `JSON:API` document in the request body to a `JSONAPIPlug.Document` struct,\nnotmalizes it and stores params in the `Plug.Conn` private assigns for later use.","ref":"JSONAPIPlug.Plug.Params.html"},{"type":"module","title":"JSONAPIPlug.Plug.QueryParam","doc":"JSON:API Query Paramter parser plug\n\nPlug for parsing a JSON:API query parameter via a `JSONAPI.QueryParser` implementation.\n\nIt takes an atom corresponding to the query parameter name to parse as its only option\nand stores the returned value in the `JSONAPIPlug` struct stored in `Plug.Conn` private\nassigns under the `jsonapi_plug` key.","ref":"JSONAPIPlug.Plug.QueryParam.html"},{"type":"module","title":"JSONAPIPlug.Plug.ResponseContentType","doc":"Plug for setting the response content type\n\nRegisters a before send function that sets the `JSON:API` content type on responses unless a response\ncontent type has already been set on the connection.","ref":"JSONAPIPlug.Plug.ResponseContentType.html"},{"type":"module","title":"JSONAPIPlug.Normalizer.Ecto","doc":"JSON:API Document normalizer implementation for Ecto\n\nTranslates `JSON:API` resources to an Ecto friendly format in conn params.\nDeserialization produces attributes and relationships in a way that directly translates\nto attributes that can be passed to an `Ecto.Changeset` for validation and later to\n`Ecto.Repo` for database operations.","ref":"JSONAPIPlug.Normalizer.Ecto.html"},{"type":"module","title":"JSONAPIPlug.QueryParser.Ecto.Fields","doc":"JSON:API 'fields' query parameter parser implementation for Ecto\n\nExpects `include` parameter to in the [JSON:API fields](https://jsonapi.org/format/#fetching-sparse-fieldsets)\nformat and converts the specification format to a map of fields list format for ease of\nuse with `select` option to `Ecto.Repo` functions.","ref":"JSONAPIPlug.QueryParser.Ecto.Fields.html"},{"type":"module","title":"JSONAPIPlug.QueryParser.Ecto.Include","doc":"JSON:API 'include' query parameter parser implementation for Ecto\n\nExpects `include` parameter to be in the [JSON:API include](https://jsonapi.org/format/#fetching-includes)\nformat and converts them to Ecto `preload` option to `Ecto.Repo` functions.","ref":"JSONAPIPlug.QueryParser.Ecto.Include.html"},{"type":"function","title":"JSONAPIPlug.QueryParser.Ecto.Include.check_relationship_include/6","doc":"","ref":"JSONAPIPlug.QueryParser.Ecto.Include.html#check_relationship_include/6"},{"type":"module","title":"JSONAPIPlug.QueryParser.Ecto.Sort","doc":"JSON:API 'sort' query parameter parser implementation for Ecto\n\nExpects sort parameters to be in the *recommended* [JSON:API sort](https://jsonapi.org/format/#fetching-sorting)\nformat and converts them to Ecto `order_by` format for ease of use.","ref":"JSONAPIPlug.QueryParser.Ecto.Sort.html"},{"type":"behaviour","title":"JSONAPIPlug.QueryParser","doc":"Parses a JSON:API query parameter to a user defined format.\n\nYou can implement a custom format for `JSON:API` query parameters that do not define\na standard format: filter, page and sort.\n\nTo customize the default parser implement a module adopting the `JSONAPIPlug.QueryParser`\nbehaviour for your sort format:\n\n```elixir\ndefmodule MyApp.API.QueryParsers.Sort do\n  ...\n  @behaviour JSONAPIPlug.QueryParser\n\n  # Transforms the query parameter value to a user defined format\n  @impl JSONAPIPlug.QueryParser\n  def parse(jsonapi_plug, sort) do\n    ...my sort query parameter parsing logic...\n  end\nend\n```\n\nand configure it in your API configuration under the `parsers` key.\n\n```elixir\nconfig :my_app, MyApp.API,\n  parsers: [\n    filter: MyApp.API.QueryParser.Filter,\n    page: MyApp.API.QueryParser.Page,\n    sort: MyApp.API.QueryParser.Sort\n  ]\n```\n\nThe parsers take the query parameter value as input and its return value is placed under an\nattribute with the same name as the query parameter in the `JSONAPIPlug` structure in the conn\n`private` assigns so that you can retrieve it and use it in your application logic.\n\nYou can return a standard `JSON:API` error to the client by raising `JSONAPIPlug.Exceptions.InvalidQuery`\nexception at any point in your parser code.","ref":"JSONAPIPlug.QueryParser.html"},{"type":"callback","title":"JSONAPIPlug.QueryParser.parse/2","doc":"Transforms the value of the JSON:API 'filter' query parameter","ref":"JSONAPIPlug.QueryParser.html#c:parse/2"},{"type":"module","title":"JSONAPIPlug.QueryParser.Filter","doc":"JSON:API 'filter' query parameter parser\n\nSince the specification does not define the format for the [JSON:API filter](http://jsonapi.org/format/#fetching-filtering)\nparameter, the default implementation just returns the value of 'filter' as is.","ref":"JSONAPIPlug.QueryParser.Filter.html"},{"type":"module","title":"JSONAPIPlug.QueryParser.Page","doc":"JSON:API 'page' query parameter parser\n\nSince the specification does not define the format for the [JSON:API page](http://jsonapi.org/format/#fetching-filtering)\nparameter, the default implementation just returns the value of 'page' as is.","ref":"JSONAPIPlug.QueryParser.Page.html"},{"type":"behaviour","title":"JSONAPIPlug.Normalizer","doc":"Transforms user data to and from a `JSON:API` Document.\n\nThe default implementation transforms `JSON:API`documents in requests to an ecto\nfriendly format and expects `Ecto.Schema` instances when rendering data in responses.\nThe data it produces is stored under the `:params` key of the `JSONAPIPlug` struct\nthat will be stored in the `Plug.Conn` private assign `:jsonapi_plug`.\n\nYou can customize normalization to convert your application data to and from\nthe `JSONAPIPlug.Document` data structure by providing an implementation of\nthe `JSONAPIPlug.Normalizer` behaviour.\n\n```elixir\ndefmodule MyApp.API.Normalizer\n  ...\n\n  @behaviour JSONAPIPlug.Normalizer\n\n  ...\nend\n```\n\nand by configuring it in your api configuration:\n\n```elixir\nconfig :my_app, MyApp.API, normalizer: MyApp.API.Normalizer\n```\n\nYou can return an error during parsing by raising `JSONAPIPlug.Exceptions.InvalidDocument` at\nany point in your normalizer code.","ref":"JSONAPIPlug.Normalizer.html"},{"type":"function","title":"JSONAPIPlug.Normalizer.denormalize/3","doc":"Transforms a JSON:API Document user data","ref":"JSONAPIPlug.Normalizer.html#denormalize/3"},{"type":"callback","title":"JSONAPIPlug.Normalizer.denormalize_attribute/3","doc":"","ref":"JSONAPIPlug.Normalizer.html#c:denormalize_attribute/3"},{"type":"callback","title":"JSONAPIPlug.Normalizer.denormalize_relationship/4","doc":"","ref":"JSONAPIPlug.Normalizer.html#c:denormalize_relationship/4"},{"type":"function","title":"JSONAPIPlug.Normalizer.normalize/5","doc":"Transforms user data into a JSON:API Document","ref":"JSONAPIPlug.Normalizer.html#normalize/5"},{"type":"callback","title":"JSONAPIPlug.Normalizer.resource_params/0","doc":"","ref":"JSONAPIPlug.Normalizer.html#c:resource_params/0"},{"type":"type","title":"JSONAPIPlug.Normalizer.params/0","doc":"","ref":"JSONAPIPlug.Normalizer.html#t:params/0"},{"type":"type","title":"JSONAPIPlug.Normalizer.t/0","doc":"","ref":"JSONAPIPlug.Normalizer.html#t:t/0"},{"type":"type","title":"JSONAPIPlug.Normalizer.value/0","doc":"","ref":"JSONAPIPlug.Normalizer.html#t:value/0"},{"type":"behaviour","title":"JSONAPIPlug.Pagination","doc":"JSON:API Pagination strategy\n\nhttps://jsonapi.org/format/#fetching-pagination\n\nPagination links can be generated by implementing a module conforming to the\n`JSONAPIPlug.Pagination` behavior and configuring the `pagination` of your API module:\n\n```elixir\ndefmodule MyApp.MyController do\n  plug JSONAPIPlug.Plug, api: MyApp.MyApi, resource: MyApp.MyResource\nend\n```\n\n```elixir\nconfig :my_app, MyApp.API, pagination: MyApp.MyPagination\n```\n\nActual pagination needs to be handled by your application and is outside the scope of this library.\n\nLinks can be generated using the `JSONAPIPlug.page` information stored in the connection `jsonapi_plug` private field\nand by passing additional information to your pagination module by passing `options` from your controller.\n\nSee the tests for an example implementation of page based pagination strategy.","ref":"JSONAPIPlug.Pagination.html"},{"type":"callback","title":"JSONAPIPlug.Pagination.paginate/4","doc":"","ref":"JSONAPIPlug.Pagination.html#c:paginate/4"},{"type":"function","title":"JSONAPIPlug.Pagination.url_for/3","doc":"","ref":"JSONAPIPlug.Pagination.html#url_for/3"},{"type":"type","title":"JSONAPIPlug.Pagination.link/0","doc":"","ref":"JSONAPIPlug.Pagination.html#t:link/0"},{"type":"type","title":"JSONAPIPlug.Pagination.links/0","doc":"","ref":"JSONAPIPlug.Pagination.html#t:links/0"},{"type":"type","title":"JSONAPIPlug.Pagination.options/0","doc":"","ref":"JSONAPIPlug.Pagination.html#t:options/0"},{"type":"type","title":"JSONAPIPlug.Pagination.params/0","doc":"","ref":"JSONAPIPlug.Pagination.html#t:params/0"},{"type":"type","title":"JSONAPIPlug.Pagination.t/0","doc":"","ref":"JSONAPIPlug.Pagination.html#t:t/0"},{"type":"exception","title":"JSONAPIPlug.Exceptions.InvalidDocument","doc":"Defines a generic exception for when an invalid document is received.","ref":"JSONAPIPlug.Exceptions.InvalidDocument.html"},{"type":"exception","title":"JSONAPIPlug.Exceptions.InvalidHeader","doc":"Defines a generic exception for when an invalid header is received.","ref":"JSONAPIPlug.Exceptions.InvalidHeader.html"},{"type":"exception","title":"JSONAPIPlug.Exceptions.InvalidQuery","doc":"Defines a generic exception for when an invalid query parameter is received.","ref":"JSONAPIPlug.Exceptions.InvalidQuery.html"},{"type":"extras","title":"JSON:API library for Plug and Phoenix applications","doc":"# JSON:API library for Plug and Phoenix applications\n\nServer library to build [JSON:API](http://jsonapi.org) compliant REST APIs.","ref":"readme.html"},{"type":"extras","title":"JSON:API Support - JSON:API library for Plug and Phoenix applications","doc":"This library currently implements version `1.0` of the [JSON:API][json:api] specification.\nHowever, to support resource creation alongside included resources, `JSON:API 1.1` `lid` is supported.","ref":"readme.html#json-api-support"},{"type":"extras","title":"Documentation - JSON:API library for Plug and Phoenix applications","doc":"- Full documentation can be found on [hexdocs.pm][jsonapi_plug-hexdocs]\n\nFor the `JSON:API` specification documentation as well as other client and server implementations:\n\n- [JSON:API][json:api]\n- [JSON:API Specification (v1.0)][json:api-1.0]\n- [JSON:API Specification (v1.1)][json:api-1.1]","ref":"readme.html#documentation"},{"type":"extras","title":"Quickstart - JSON:API library for Plug and Phoenix applications","doc":"","ref":"readme.html#quickstart"},{"type":"extras","title":"Installation - JSON:API library for Plug and Phoenix applications","doc":"Add the following line to your `mix.deps` file with the desired version to install `jsonapi_plug`.\n\n```elixir\ndefp deps do [\n  ...\n  {:jsonapi_plug, \"~> 2.0\"}\n  ...\n]\n```","ref":"readme.html#installation"},{"type":"extras","title":"Upgrading - JSON:API library for Plug and Phoenix applications","doc":"Upgrading the library should be safe for minor and patch vesion upgrades, it's still a good practice\nto check the [changelog][changelog] for more information on the release content. For major version\nupgrades, breaking changes should be expected, and the [upgrade guide][upgrade] contains instructions\non upgrading between major releases of the library.","ref":"readme.html#upgrading"},{"type":"extras","title":"Configuration - JSON:API library for Plug and Phoenix applications","doc":"You start by declaring one or more APIs. APIs are collections of endpoints that\nshare a common configuration:\n\n```elixir\ndefmodule MyApp.API do\n  use JSONAPIPlug.API, otp_app: :my_app\nend\n```\n\nSee the `JSONAPIPlug.API` module documentation to learn how to customize your APIs\nvia application configuration of your app.","ref":"readme.html#configuration"},{"type":"extras","title":"Resources - JSON:API library for Plug and Phoenix applications","doc":"To start accept requests and serving responses, you need to define `JSON:API` resources.\nResources can be any struct `@derive`-ing the `JSONAPIPlug.Resource` protocol:\n\n```elixir\ndefmodule MyApp.Post do\n  use Ecto.Schema\n\n  @type t :: %__MODULE__{id: pos_integer(), body: String.t(), title: String.t()}\n\n  @derive {\n    JSONAPIPlug.Resource,\n    type: \"post\",\n    attributes: [:title, :text, :excerpt]\n  }\n  schema \"posts\" do\n    field :title\n    field :text\n  end\n\n  ...\nend\n```\n\nSee `JSONAPIPlug.Resource` for the complete documentation of options you can pass to `@derive`,\nincluding how to control serialization and deserialization, and add related resources.\n\nAlso, three optional protocols are available to allow further customization of resources.\n\n- Resource attribute custom serialization and deserialization: `JSONAPIPlug.Resource.Attribute`\n- Resource link generation: `JSONAPIPlug.Resource.Links`\n- Resource link generation: `JSONAPIPlug.Resource.Meta`","ref":"readme.html#resources"},{"type":"extras","title":"Usage with Phoenix - JSON:API library for Plug and Phoenix applications","doc":"To serve JSON:API resources in Phoenix, you need to define routes in your router:\n\n```elixir\ndefmodule MyAppWeb.Router do\n  ...\n  resource \"/posts\", MyApp.PostsController, only: [:create, :index, :show]\n  patch \"/posts/:id\", MyApp.PostsController, :update\nend\n```\n\nIn order to parse `JSON:API` requests from clients you need to add the `JSONAPIPlug.Plug` plug to each of your\nphoenix controllers handling requests for a specific resource. This will take care of ensuring `JSON:API` request\ncompliance and will return errors for malformed requests.\n\nWhen a valid request processed, the `:jsonapi_plug` `Plug.Conn` private field will be populated in the controller.\n\nYou can learn all about advanced request handling, including custom filtering, relationships inclusion, pagination\nand sparse fields support by reading the `JSONAPIPlug.Plug` module documentation.\n\nOnce you receive a request in your controller and load data, you just call render to send a response:\n\n```elixir\n  defmodule MyAppWeb.PostsController do\n    ...\n    plug JSONAPIPlug.Plug, api: MyApp.API, path: \"posts\", resource: MyApp.Post\n    ...\n\n    def create(%Conn{private: %{jsonapi_plug: %JSONAPIPlug{} = jsonapi_plug}} = conn, params) do\n      post = ...create a post using jsonapi_plug parsed parameters...\n      render(conn, \"create.json\", %{data: post})\n    end\n\n    def index(%Conn{private: %{jsonapi_plug: %JSONAPIPlug{} = jsonapi_plug}} = conn, _params) do\n      posts = ...load data using jsonapi_plug parsed parameters...\n      render(conn, \"index.json\", %{data: post})\n    end\n\n    def show(%Conn{private: %{jsonapi_plug: %JSONAPIPlug{} = jsonapi_plug}} = conn, _params) do\n      post = ...load data using jsonapi_plug parsed parameters...\n      render(conn, \"show.json\", %{data: post})\n    end\n\n    def udate(%Conn{private: %{jsonapi_plug: %JSONAPIPlug{} = jsonapi_plug}} = conn, params) do\n      post = ...update a post using jsonapi_plug parsed parameters...\n      render(conn, \"update.json\", %{data: post})\n    end\n  end\n```\n\nFor phoenix to dispatch this for rendering you need to add this code to your `MyAppWeb` module:\n\n```elixir\ndef MyAppWeb do\n ...\n\n  def json_api do\n    quote do\n      use JSONAPIPlug.Phoenix.Component\n    end\n  end\n\n...\nend\n```\n\nand define a corresponding rendering template module:\n\n```elixir\ndefmodule SibillWeb.PostsJSON do\n  @moduledoc false\n\n  use MyAppWeb, :json_api\nend\n```\n\nalternatively you can skip these steps by calling `JSONAPIPlug.render/4` directly instead of `render/3` in you controller:\n\n```elixir\n...\n    def show(%Conn{private: %{jsonapi_plug: %JSONAPIPlug{} = jsonapi_plug}} = conn, _params) do\n      post = ...load data using jsonapi_plug parsed parameters...\n      JSONAPIPlug.render(conn, post \\\\ nil, meta \\\\ nil, options \\\\ [])\n    end\n...\n```","ref":"readme.html#usage-with-phoenix"},{"type":"extras","title":"Usage with Plug - JSON:API library for Plug and Phoenix applications","doc":"If you have a `Plug` application, assuming you already set up routing you can call `JSONAPIPlug.render/4` in your\npipeline to generate a `JSONAPI.Document` with your data for the client.\n\n```elixir\nJSONAPIPlug.render(conn, post)\n|> Jason.encode!()\n```\n\nRender returns a `JSONAPI.Document`, that is serializable to JSON via `Jason`.","ref":"readme.html#usage-with-plug"},{"type":"extras","title":"Contributing - JSON:API library for Plug and Phoenix applications","doc":"- This project was born as a fork of the [jsonapi][jsonapi-fork]\nlibrary but has since been completely rewritten and is now a completely different project.\n- PRs for new features, bug fixes, documentation and tests are welcome\n- If you are proposing a large feature or change, please open an issue for discussion\n\n[changelog]: https://hexdocs.pm/jsonapi_plug/changelog.html\n[json:api-1.0]: https://jsonapi.org/format/1.0/\n[json:api-1.1]: https://jsonapi.org/format/1.1/\n[json:api]: https://jsonapi.org\n[jsonapi-fork]: https://github.com/beam-community/jsonapi\n[jsonapi_plug-hexdocs]: https://hexdocs.pm/jsonapi_plug\n[upgrade]: https://hexdocs.pm/jsonapi_plug/upgrading.html","ref":"readme.html#contributing"},{"type":"extras","title":"Upgrading between major versions","doc":"# Upgrading\n\nInstructions to upgrade between major versions of `JSONAPIPlug`.","ref":"upgrading.html"},{"type":"extras","title":"Upgrading from 1.x to 2.0 - Upgrading between major versions","doc":"To upgrade your application to 2.0:\n\n  1. Replace all `use JSONAPIPlug.Resource, ...options...` in your `_json.ex` files with one of:\n    - `@derive {JSONAPIPlug.Resource, ...options...}` in the module defining your structs.\n    - `require Protocol` and `Protocol.derive(JSONAPIPlug.Resource, MyStruct, ...options...)`\n      if you prefer to keep it in the `_json.ex` modules.\n    - Provide a manual implementation of `JSONAPIPlug.Resource`. This is discouraged\n      because the derivation macro generates functions for recasing and options that are tedious to\n      implement manually.\n  2. Move the `path` option from `JSONAPIPlug.Resource` options to `JSONAPIPlug.Plug` options in your controllers.\n  3. `JSONAPIPlug.Resource` attribute options `serialize` and `deserialize` now only support a boolean value.\n      Remove all function references passed to `serialize` and `deserialize` and provide an implementation\n      of the `JSONAPIPlug.Resource.Attribute` protocol for your resource to do custom serialization/deserialization.\n  4. If overridden, replace the `JSONAPIPlug.Resource.links` callback with an implementation of the\n     `JSONAPIPlug.Resource.Links` protocol for your resource to add per-resource `JSON:API` links.\n  5. If overridden, replace the `JSONAPIPlug.Resource.meta` callback with an implementation of the\n    `JSONAPIPlug.Resource.Meta` protocol for your resource to add per-resource `JSON:API` meta.\n  6. If you use phonenix, either call `JSONAPIPlug.render/5` in your controllers or add `use MyAppWeb, :json_api`\n     to your `_json.ex` modules and call `render/3`. See the README for complete instructions.\n  7. If you are sending resource `ids` in `included` for resource create requests, this is now forbidden unless\n     the `client_generated_ids` option is configured on your `JSONAPIPlug.API`. You can however use `JSON:API 1.1`\n     `lid` in relationships and included resources if you whish to create resources with included resources atomically.\n     Please note that this works even though the reported `JSON:API` version is still `1.0`, because the library still\n     does not support the full `JSON:API 1.1` specification yet, only `lid` is supported for now.","ref":"upgrading.html#upgrading-from-1-x-to-2-0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"2.0.0 \"Protocols\" (2024-11-09) - Changelog","doc":"`JSONAPIPlug` 2.0 has moved to a protocol based approach to resource definition.\nThis is the result of a big refactoring, cleaning up internals and providing a more\nstable foundation for the library moving forward. This means there are a number of\nbreaking changes that require changes to applications using `JSONAPIPlug`.\n\nSee the [upgrade guide][upgrade] in the docs for detailed upgrade instructions from 1.0.\n\n- `JSONAPIPlug.Resource` is now a protocol instead of a behaviour.\n  Using structs as resource data is now mandatory.\n- Moved `path` option from `JSONAPI.Resource` to `JSONAPIPlug.Plug`.\n- Passing functions to `JSONAPIPlug.Resource` attribute `serialize`\n  and `deserialize` to customize attribute value serialization and\n  deserialization has been replaced by `JSONAPIPlug.Resource.Attribute`.\n- Generating per-resource `JSON:API` links and meta with `JSONAPIPlug.Resource`\n  callbacks has been replaces by the `JSONAPIPlug.Resource.Links` and\n  `JSONAPIPlug.Resource.Meta` protocols.\n- Removed `links` option to `JSONAPIPlug.API`. Resource links are always generated.\n- Moved the Phoenix render function to a component module in the library, this can\n  be added to the phoenix `MyAppWeb` module and imported in the phoenix `_json.ex`\n  module via `use MyAppWeb, :jsonapi` as per phoenix conventions.\n- Enforce `client_generated_ids` option. This prevents sending ids in resources and\n  included resources when `client_generated_ids` is turned off. If you were sending\n  ids to support resource creation with included atomically, this is now supported\n  by sending `JSON:API 1.1` `lid` in relationships and included resources. This\n  is supported even though the reported jsonapi vesion is still `1.0` because the\n  library still doesn't have full `JSON:API 1.1` support. Only `lid` is supported.\n\nContributors: @lucacorti @treere","ref":"changelog.html#2-0-0-protocols-2024-11-09"},{"type":"extras","title":"1.0.7 (2024-09-23) - Changelog","doc":"- Fix case in deserialization of relationships (@treere)","ref":"changelog.html#1-0-7-2024-09-23"},{"type":"extras","title":"1.0.6 (2024-05-24) - Changelog","doc":"- Fix deserialization of many relationships (@lucacorti, @alexgolasibill)","ref":"changelog.html#1-0-6-2024-05-24"},{"type":"extras","title":"1.0.5 (2024-01-30) - Changelog","doc":"- Allow disabling generation of links for relationships and includes. (@treere)\n- Add compile time optimizations for case transformation of resource fields (@treere)\n- Add support to restrict allowed includes to `JSONAPIPlug.Plug`. (@lucacorti)","ref":"changelog.html#1-0-5-2024-01-30"},{"type":"extras","title":"1.0.4 (2023-10-23) - Changelog","doc":"- Fix deeply nested includes not always serialized correctly (@treere)","ref":"changelog.html#1-0-4-2023-10-23"},{"type":"extras","title":"1.0.3 (2023-04-28) - Changelog","doc":"- Accept id along with attributes in default sort parser implementation (@agos)\n- Remove deprecated builder_opts() usage from Plug.Builder (@lucacorti)","ref":"changelog.html#1-0-3-2023-04-28"},{"type":"extras","title":"1.0.2 (2023-04-14) - Changelog","doc":"- Cache configuration (profiling by @treere, @lucacorti)\n- Relax :nimble_options dependency (@lucacorti)","ref":"changelog.html#1-0-2-2023-04-14"},{"type":"extras","title":"1.0.1 (2023-03-15) - Changelog","doc":"- Allow nimble options 1.0 (@lucacorti)","ref":"changelog.html#1-0-1-2023-03-15"},{"type":"extras","title":"1.0.0 \"Garetto Basso\" (2023-02-18) - Changelog","doc":"First release of `jsonapi_plug`: `JSON:API` library for Plug and Phoenix applications.\n\nThis project was born as a fork of the [jsonapi](https://github.com/beam-community/jsonapi)\nlibrary but has since been completely rewritten and is a different project.\n\nWhat `jsonapi_plug` has to offer to users of Phoenix/Plug for building `JSON:API` compliant APIs:\n\n- An extensible system based on behaviours to convert `JSON:API` payloads and query parameters from/to whatever format best suits the library user. Default implementations are provided to convert payloads and query parameters to an Ecto friendly format, which is probably the most common scenario for the data source of APIs in Phoenix based applications. Behaviours are available to customize parsing and/or easily parse non-standard specified query parameters (filter, page).\n- Ability to have multiple APIs with different configurations sourced from the hosting application configuration to avoid single global configuration rigidity. This allows to serve multiple different APIs from a single Phoenix/Plug application instance.\n- A declarative resource system allowing users to control rendering, rename fields between `JSON:API` payloads and internal data, customize (de)serialization of fields values and more without writing additional business logic code.\n\nContributors: @lucacorti\n\n[upgrade]: https://hexdocs.pm/jsonapi_plug/upgrading.html","ref":"changelog.html#1-0-0-garetto-basso-2023-02-18"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}